/*******************************************************************************\
*                  Decode Apple Lisa VSROM Serial Number                        *
*       Copyright (C) 2021 by Ray Arachelian, All Rights Reserved               *
*                      https://lisaem.sunder.net                                *
*                                                                               *
*               Released under the terms of the GNU GPL 3.0                     *
*            see: https://www.gnu.org/licenses/gpl-3.0.en.html                  *
*                                                                               *
*            "Oh boy, here I go codin' again! I just love codin'"               *
*                                                                               *
\*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define OUT stdout

unsigned char vsrom[256];

int main(int argc, char *argv[])
{
  FILE *ROM;
  int i;

  #define SIZE 32

  unsigned char sn[33];

  // VSROM is 256 bytes, serial is 16 bytes, applenet is also another 16 bytes after

  #define RESET (1)
  #define VSYNC (2)
  #define BIT2  (4)
  #define BIT3  (8)
  #define CSYNC (16)
  #define VSIR  (32)
  #define HSYNC (64)
  #define SERNO (128)


  puts(" =================================================================================" );
  puts(" =                                                                               =" );
  puts(" =                  Decode Apple Lisa VSROM Serial Number                        =" );
  puts(" =       Copyright (C) 2021 by Ray Arachelian, All Rights Reserved               =" );
  puts(" =                      https://lisaem.sunder.net                                =" );
  puts(" =               Released under the terms of the GNU GPL 3.0                     =" );
  puts(" =                                                                               =" );
  puts(" =            see: https://www.gnu.org/licenses/gpl-3.0.en.html                  =" );
  puts(" =      https://lisalist2.com/index.php/topic,193.msg1485.html#msg1485           =" );
  puts(" =                                                                               =" );
  puts(" =================================================================================\n" );

  if (argc!=2) {
     fprintf(stderr,"Usage: decode-vsrom videostate.rom\n\n");
     exit(1);
  }

  ROM=fopen(argv[1],"rb");
  i=fread(vsrom,256,1,ROM);

  memset(sn,0,32);

  for (i=0; i<256; i++)
   {
    if (vsrom[i ^ 128] & SERNO) sn[i/8] |=1<<(7-(i & 7));

/*  printf("%3d:%02x RST:%d VSYNC:%d CSYNC:%d VSIR:%d HSYNC:%d SERNO:%d 2:%d 3:%d\n",i,vsrom[i],
     (vsrom[i] & RESET) ? 1:0,
     (vsrom[i] & VSYNC) ? 1:0,
     (vsrom[i] & CSYNC) ? 1:0,
     (vsrom[i] & VSIR ) ? 1:0,
     (vsrom[i] & HSYNC) ? 1:0,
     (vsrom[i] & SERNO) ? 1:0,
     (vsrom[i] & BIT2 ) ? 1:0,
     (vsrom[i] & BIT3 ) ? 1:0   ); */
   } 

  fclose(ROM);

  fprintf(OUT,"240 Serial #: ");
  for (i=0; i<32; i++) { 
     if ((i & 15)<8) fprintf(OUT,"%02x%02x ",(sn[i] & 0xf0)>>4,sn[i] & 0x0f);
     if (i==15) fprintf(OUT,          "\n260 AppleNet: ");
     //else if ((i & 7)==7) fprintf(OUT,"\n              ");
  }
  fprintf(OUT,"\n\n");

}
